"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.transformWidget = void 0;
const tools_1 = require("../tools");
const lodash = require("lodash");
/**
 * Takes any unresolved non-expression widget parameter that ends with "color"
 * and parses the color css value.
 */
function transformWidget(widget, options) {
    if (!widget.params)
        return widget;
    const colorParams = widget.params.filter(param => param.name
        && (param.name.toLowerCase().endsWith('color') || param.name.indexOf('Color') > 0)
        && !param.resolved
        && param.type != 'expression');
    lodash.pullAll(widget.params, colorParams);
    for (let param of colorParams) {
        widget.params.push({
            class: 'param',
            name: param.name,
            type: 'expression',
            value: (0, tools_1.parseStyleColor)(param.value.toString()),
            resolved: true
        });
    }
    (0, tools_1.applyOnDescendants)(widget, descendant => transformWidget(descendant, options));
    return widget;
}
exports.transformWidget = transformWidget;
