"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.transformWidget = void 0;
const lodash_1 = require("lodash");
const tools_1 = require("../tools");
/**
 * This plugin processes the **display** parameters.
 *
 * @param widget the widget tree to process
 * @param options the flutter-view options
 * @return the transformed widget tree
 */
function transformWidget(widget, options) {
    const childrenParam = (0, tools_1.findParam)(widget, 'children', true);
    if (childrenParam) {
        const params = [];
        const children = childrenParam.value;
        // we cannot remove while iterating the children, so we do this in two steps:
        // 1) find all the children to be removed
        for (let child of children) {
            const displayParam = (0, tools_1.findAndRemoveParam)(child, 'display', {
                includeResolved: true,
                includeExpressions: true
            });
            if (displayParam && displayParam.value == 'none') {
                params.push({
                    name: displayParam.value.toString(),
                    value: child
                });
            }
        }
        // 2) remove each param's widget from the children param widget
        for (let param of params) {
            (0, lodash_1.pull)(children, param.value);
        }
    }
    const childParam = (0, tools_1.findParam)(widget, 'child', true);
    if (childParam) {
        const child = childParam.value;
        const displayParam = (0, tools_1.findAndRemoveParam)(child, 'display');
        if (displayParam && displayParam.value == 'none') {
            (0, tools_1.findAndRemoveParam)(widget, 'child', {
                includeResolved: true,
                includeExpressions: true
            });
        }
    }
    (0, tools_1.applyOnDescendants)(widget, descendant => transformWidget(descendant, options));
    return widget;
}
exports.transformWidget = transformWidget;
