"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.transformWidget = void 0;
const tools_1 = require("../tools");
const change_case_1 = require("change-case");
function transformWidget(widget, options) {
    const widthParam = (0, tools_1.findAndRemoveParam)(widget, 'width', {
        includeExpressions: false,
        includeResolved: true
    });
    const heightParam = (0, tools_1.findAndRemoveParam)(widget, 'height', {
        includeExpressions: false,
        includeResolved: true
    });
    const sizeParam = (0, tools_1.findAndRemoveParam)(widget, 'size', {
        includeExpressions: false,
        includeResolved: true
    });
    const fitParam = (0, tools_1.findAndRemoveParam)(widget, 'fit', {
        includeExpressions: false,
        includeResolved: true
    });
    const paddingParam = (0, tools_1.findAndRemoveParam)(widget, 'padding', {
        includeExpressions: false,
        includeResolved: true
    });
    const paddingTopParam = (0, tools_1.findAndRemoveParam)(widget, 'paddingTop');
    const paddingRightParam = (0, tools_1.findAndRemoveParam)(widget, 'paddingRight');
    const paddingBottomParam = (0, tools_1.findAndRemoveParam)(widget, 'paddingBottom');
    const paddingLeftParam = (0, tools_1.findAndRemoveParam)(widget, 'paddingLeft');
    const marginParam = (0, tools_1.findAndRemoveParam)(widget, 'margin', {
        includeExpressions: false,
        includeResolved: true
    });
    const marginTopParam = (0, tools_1.findAndRemoveParam)(widget, 'marginTop');
    const marginRightParam = (0, tools_1.findAndRemoveParam)(widget, 'marginRight');
    const marginBottomParam = (0, tools_1.findAndRemoveParam)(widget, 'marginBottom');
    const marginLeftParam = (0, tools_1.findAndRemoveParam)(widget, 'marginLeft');
    const borderRadiusParam = (0, tools_1.findAndRemoveParam)(widget, 'borderRadius', {
        includeExpressions: false,
        includeResolved: true
    });
    const alignmentParam = (0, tools_1.findAndRemoveParam)(widget, 'alignment', {
        includeExpressions: false,
        includeResolved: true
    });
    // dimensions
    if (sizeParam && sizeParam.value) {
        widget.params.push({
            class: 'param',
            name: 'size',
            type: 'expression',
            value: (0, tools_1.parseStyleDoubleValue)(sizeParam.value.toString()),
            resolved: true
        });
    }
    if (widthParam && widthParam.value) {
        widget.params.push({
            class: 'param',
            name: 'width',
            type: 'expression',
            value: (0, tools_1.parseStyleDoubleValue)(widthParam.value.toString()),
            resolved: true
        });
    }
    if (heightParam && heightParam.value) {
        widget.params.push({
            class: 'param',
            name: 'height',
            type: 'expression',
            value: (0, tools_1.parseStyleDoubleValue)(heightParam.value.toString()),
            resolved: true
        });
    }
    if (fitParam && fitParam.value) {
        widget.params.push({
            class: 'param',
            name: 'fit',
            type: 'expression',
            value: `BoxFit.${(0, change_case_1.camelCase)(fitParam.value.toString())}`,
            resolved: true
        });
    }
    // alignment
    if (alignmentParam && alignmentParam.value) {
        widget.params.push({
            class: 'param',
            name: 'alignment',
            type: 'expression',
            value: `Alignment.${(0, change_case_1.camelCase)(alignmentParam.value.toString())}`,
            resolved: true
        });
    }
    // padding
    let paddings = {};
    if (paddingParam && paddingParam.value) {
        paddings = (0, tools_1.parseTRBLStyle)(paddingParam.value.toString());
    }
    if (paddingTopParam && paddingTopParam.value) {
        paddings.top = paddingTopParam.value.toString();
    }
    if (paddingRightParam && paddingRightParam.value) {
        paddings.right = paddingRightParam.value.toString();
    }
    if (paddingBottomParam && paddingBottomParam.value) {
        paddings.bottom = paddingBottomParam.value.toString();
    }
    if (paddingLeftParam && paddingLeftParam.value) {
        paddings.left = paddingLeftParam.value.toString();
    }
    if (Object.keys(paddings).length > 0) {
        const params = [];
        if (paddings.top)
            params.push('top: ' + (0, tools_1.parseStyleDoubleValue)(paddings.top));
        if (paddings.right)
            params.push('right: ' + (0, tools_1.parseStyleDoubleValue)(paddings.right));
        if (paddings.bottom)
            params.push('bottom: ' + (0, tools_1.parseStyleDoubleValue)(paddings.bottom));
        if (paddings.left)
            params.push('left: ' + (0, tools_1.parseStyleDoubleValue)(paddings.left));
        const inset = `EdgeInsets.only(${params.join(', ')})`;
        widget.params.push({
            class: 'param',
            name: 'padding',
            type: 'expression',
            value: inset,
            resolved: true
        });
    }
    // margin
    let margins = {};
    if (marginParam && marginParam.value) {
        margins = (0, tools_1.parseTRBLStyle)(marginParam.value.toString());
    }
    if (marginTopParam && marginTopParam.value) {
        margins.top = marginTopParam.value.toString();
    }
    if (marginRightParam && marginRightParam.value) {
        margins.right = marginRightParam.value.toString();
    }
    if (marginBottomParam && marginBottomParam.value) {
        margins.bottom = marginBottomParam.value.toString();
    }
    if (marginLeftParam && marginLeftParam.value) {
        margins.left = marginLeftParam.value.toString();
    }
    if (Object.keys(margins).length > 0) {
        const params = [];
        if (margins.top)
            params.push('top: ' + (0, tools_1.parseStyleDoubleValue)(margins.top));
        if (margins.right)
            params.push('right: ' + (0, tools_1.parseStyleDoubleValue)(margins.right));
        if (margins.bottom)
            params.push('bottom: ' + (0, tools_1.parseStyleDoubleValue)(margins.bottom));
        if (margins.left)
            params.push('left: ' + (0, tools_1.parseStyleDoubleValue)(margins.left));
        const inset = `EdgeInsets.only(${params.join(', ')})`;
        widget.params.push({
            class: 'param',
            name: 'margin',
            type: 'expression',
            value: inset,
            resolved: true
        });
    }
    // border radius
    if (borderRadiusParam && borderRadiusParam.value)
        widget.params.push({
            class: 'param',
            name: 'borderRadius',
            type: 'expression',
            value: (0, tools_1.toBorderRadiusCode)(borderRadiusParam),
            resolved: true
        });
    // also apply the plugin to the rest of the widget tree of this widget
    (0, tools_1.applyOnDescendants)(widget, descendant => transformWidget(descendant, options));
    return widget;
}
exports.transformWidget = transformWidget;
